@@ -60,6 +60,7 @@ class INTEGRALwrapper(object):
         total_scw_list = sorted(in_total_scw_list)
 
         self.product = arguments['product']
+        local_arguments = arguments.copy()
 
         if len(total_scw_list) > s_max:
             ind_max = int(len(total_scw_list) / s_max)
@@ -69,6 +70,23 @@ class INTEGRALwrapper(object):
         else:
             scw_lists = [total_scw_list]
 
+        def get_revs(loc_scw_list):
+            return np.array(sorted(list(set([int(a[0:4]) for a in loc_scw_list]))))
+
+        ## I split a list that contains revolutions before and after 1626 (start of validity of OSA11.X
+        new_scw_lists = []
+        for ss in scw_lists:
+            revs = get_revs(ss)
+            if revs.min() < 1626 and revs.max() >= 1626:
+                s1 = [s for s in ss if int(s[0:4]) < 1626]
+                s2 = [s for s in ss if int(s[0:4]) >= 1626]
+                new_scw_lists.append(s1)
+                new_scw_lists.append(s2)
+            else:
+                new_scw_lists.append(ss)
+        scw_lists = new_scw_lists
+        ##
+
         self.all_data = []
         tot_num = 0
 
@@ -98,8 +116,14 @@ class INTEGRALwrapper(object):
 
                 if data is None or not _disp.is_failed():
                     if not _disp.is_submitted:
+                        revs = get_revs(scw_list)
+                        # Force 0SA10.2 for ISGRI before rev. 1626
+                        if revs.min() < 1626 and 'isgri' in arguments['product']:
+                            local_arguments['osa_version'] = 'OSA10.2'
+                        else:
+                            local_arguments['osa_version'] = arguments['osa_version']
                         scw_list_str = ",".join([s for s in sorted(set(scw_list))])
-                        data = _disp.get_product(scw_list=scw_list_str, **arguments)
+                        data = _disp.get_product(scw_list=scw_list_str, **local_arguments)
                     else:
                         _disp.poll()
 
@@ -161,10 +185,14 @@ class INTEGRALwrapper(object):
         list_hdus = [dd.mosaic_image_0_mosaic.to_fits_hdu_list() for dd in self.all_data]
         summed_mosaic = mosaic.mosaic_list(list_hdus, pixels=projection, mock=False)
 
-        sources = [dd.dispatcher_catalog_1.table[np.logical_and(
-            dd.dispatcher_catalog_1.table['significance'] >= detection_threshold,
-            dd.dispatcher_catalog_1.table['ISGRI_FLAG'] > 0)]
-                   for dd in self.all_data]
+        sources = []
+        for dd in self.all_data:
+            tab = dd.dispatcher_catalog_1.table
+            if len(tab) > 0:
+                ind = np.logical_and( tab['significance'] >= detection_threshold, tab['ISGRI_FLAG'] > 0)
+                if np.sum(ind) > 0:
+                    sources.append(tab[ind])
+
         # If there are new sources, the tyoe is 'object', we need to replace it
         for ss in sources:
             ss.replace_column('ERR_RAD', ss.columns['ERR_RAD'].astype(np.float64))
@@ -172,19 +200,25 @@ class INTEGRALwrapper(object):
         # We stack the table but take just the first occurrence of each source
         stacked_table_known = table.unique(table.vstack(sources, join_type='exact'), keys='src_names')
 
-        new_sources_list = [dd.dispatcher_catalog_1.table[dd.dispatcher_catalog_1.table['ISGRI_FLAG'] == 0] for dd in
-                            self.all_data]
+        new_sources_list = [dd.dispatcher_catalog_1.table[dd.dispatcher_catalog_1.table['ISGRI_FLAG'] == 0]
+                            for dd in self.all_data]
         n_new = 1
         for ss in new_sources_list:
             if len(ss) == 0:
-                new_sources_list.remove(ss)
                 continue
+            ss.replace_column('ERR_RAD', ss.columns['ERR_RAD'].astype(np.float64))
+            ss.replace_column('src_names', ss.columns['src_names'].astype(np.dtype('U6')))
             for row in ss:
                 row['src_names'] = 'NEW_%02d' % n_new
                 row['ERR_RAD'] = -1
                 n_new += 1
-            ss.replace_column('ERR_RAD', ss.columns['ERR_RAD'].astype(np.float64))
 
+        for ss in new_sources_list:
+            if len(ss) == 0:
+                new_sources_list.remove(ss)
+
+        self.__log.debug(new_sources_list)
+        self.__log.debug(stacked_table_known)
         stacked_table = table.unique(table.vstack(new_sources_list + [stacked_table_known], join_type='exact'),
                                      keys='src_names')
 
@@ -206,8 +240,19 @@ class INTEGRALwrapper(object):
 
         self.summed_data=summed_data
         self.compute_fluxes()
+        self.display_sources()
         return summed_data
 
+    def display_sources(self):
+        out_str = 'source_name RA DEC Flux Flux_err\n-----------------------------------'
+        self.__log.info(out_str)
+
+        for ss in self.summed_data.dispatcher_catalog_1.table:
+            out_str = '%s %.3f %.3f %.4f %.4f' % (ss['src_names'], ss['ra'], ss['dec'], ss['FLUX'],
+                                                    ss['FLUX_ERR'])
+            self.__log.info(out_str)
+
+
     def compute_fluxes(self):
 
         if not hasattr(self, 'summed_data'):
@@ -359,13 +404,27 @@ class INTEGRALwrapper(object):
         dy = np.sqrt(dy ** 2 + (y * systematic_fraction) ** 2)
         ind = np.logical_and(np.isfinite(y), np.isfinite(dy))
         ind = np.logical_and(ind, dy > 0)
-        return x[ind], y[ind], dy[ind]
+
+        #This could only be valid for ISGRI
+        try:
+            dt_lc =  hdu.data['XAX_E']
+            INTEGRALwrapper.__log.debug('Get time bin directly from light curve')
+        except:
+            timedel = hdu.header['TIMEDEL']
+            timepix = hdu.header['TIMEPIXR']
+            t_lc = hdu.data['TIME'] + (0.5 - timepix) * timedel
+            dt_lc = t_lc.copy() * 0.0 + timedel / 2
+            for i in range(len(t_lc) - 1):
+                dt_lc[i + 1] = min(timedel / 2, t_lc[i + 1] - t_lc[i] - dt_lc[i])
+            INTEGRALwrapper.__log.debug('Computed time bin from TIMEDEL')
+
+        return x[ind], dt_lc[ind], y[ind], dy[ind]
 
 
     @staticmethod
     def plot_lc(combined_lc, source_name, systematic_fraction=0, ng_sig_limit=3, find_excesses=False):
         from scipy import stats
-        x,y,dy = INTEGRALwrapper.get_lc(combined_lc, source_name, systematic_fraction)
+        x,dx,y,dy = INTEGRALwrapper.get_lc(combined_lc, source_name, systematic_fraction)
 
         meany = np.sum(y / dy ** 2) / np.sum(1. / dy ** 2)
         err_mean = np.sum(1 / dy ** 2)
@@ -373,7 +432,7 @@ class INTEGRALwrapper(object):
         std_dev = np.std(y)
 
         fig = plt.figure()
-        _ = plt.errorbar(x, y, yerr=dy, marker='o', capsize=0, linestyle='', label='Lightcurve')
+        _ = plt.errorbar(x, y, xerr=dx, yerr=dy, marker='o', capsize=0, linestyle='', label='Lightcurve')
         _ = plt.axhline(meany, color='green', linewidth=3)
         _ = plt.xlabel('Time [IJD]')
         _ = plt.ylabel('Rate')
